'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * Copyright 2015 Urban Airship Inc. and Contributors. Subject to the LICENSE
 * file at the top-level directory of this distribution and at
 * https://github.com/urbanairship/frock/blob/master/LICENSE
 */
try {
  require('babel-polyfill');
} catch (e) {
  // babel polyfill throws if it's ever included in any other module
}

var _require = require('events'),
    EventEmitter = _require.EventEmitter;

var commuter = require('commuter');
var arrayify = require('arrify');
var bole = require('bole');

var createCores = require('./cores');
var createSocketServer = require('./core-socketserver');
var createHttpServer = require('./core-httpserver');
var createHandlerRegister = require('./register-handler');
var createDbRegister = require('./register-db');
var createConfigRegister = require('./register-config');
var pkg = require('../package.json');

var log = bole('frock/index');

module.exports = createFrockInstance;

function createFrockInstance() {
  var _config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var argv = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var pwd = argv.pwd;

  var frock = new EventEmitter();
  var handlers = createHandlerRegister(pwd);
  var servers = [];
  var dbs = createDbRegister(pwd);
  var configs = createConfigRegister();

  var firstRun = true;

  frock.argv = argv;
  frock.pwd = pwd;
  frock.version = pkg.version;

  frock.run = run;
  frock.stop = stop;
  frock.reload = reload;

  frock.dbs = dbs;
  frock.handlers = handlers;

  frock.logger = bole;
  frock.router = commuter;

  configs.register(_config);

  return frock;

  function run() {
    var ready = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : noop;

    var config = configs.get(0);
    var count = 0;
    var errors = [];

    var httpServers = config.servers || [];
    var socketServers = config.sockets || [];

    // perform first-run only tasks
    if (firstRun) {
      firstRun = false;

      createCores(frock);
    }

    // configure db if requested
    if (config.db) {
      dbs._updatePath(config.db.path);
    }

    socketServers.forEach(function (socketConfig) {
      createSocketServer(frock, socketConfig, config, done);
    });

    httpServers.forEach(function (serverConfig) {
      createHttpServer(frock, serverConfig, config, done);
    });

    function done(errs, server) {
      ++count;

      if (errs) {
        errors.push.apply(errors, arrayify(errs));
      } else if (server) {
        servers.push(server);
      }

      if (count >= httpServers.length + socketServers.length) {
        frock.emit('run');
        ready(errors.length ? errors : null);
      }
    }
  }

  function reload(_config) {
    var ready = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : noop;

    log.info('reloading');

    if (_config && (typeof _config === 'undefined' ? 'undefined' : _typeof(_config)) === 'object') {
      log.debug('replacing config', _config);
      configs.register(_config);
    } else if (_config && typeof _config === 'function') {
      // support passing ready as the first param
      ready = _config;
    }

    stop(true, function () {
      run(function (err) {
        frock.emit('reload');
        ready(err);
      });
    });
  }

  function stop() {
    var hot = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
    var ready = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : noop;

    // support passing ready as the first param
    if (typeof hot === 'function') {
      ready = hot;
      hot = false;
    }

    var serverCount = 0;

    log.info('shutting down');

    servers.forEach(function (server) {
      var handlerCount = 0;

      log.debug(server.port + ': removing ' + server.handlers.length + ' handlers');

      server.handlers.forEach(function (handler, index) {
        log.debug('ending handler ' + index);

        if (handler.end) {
          handler.end(innerDone);
        } else {
          log.error('handler ' + index + ' did not have a .end function');
          innerDone();
        }

        function innerDone() {
          ++handlerCount;

          log.debug('handler ended ' + index);

          if (handlerCount >= server.handlers.length) {
            log.debug('no handlers remain, closing server ' + server.port);
            server.handlers.splice(0, server.handlers.length);

            // destroy all middlewares
            if (server.middlewares && server.middlewares.length) {
              server.middlewares.forEach(function (middleware) {
                return middleware.end();
              });
              server.middlewares.splice(0, server.middlewares.length);
            }

            server.server.destroy(done);
          }
        }
      });
    });

    function done() {
      ++serverCount;

      if (serverCount >= servers.length) {
        log.debug('servers stopped');
        servers.splice(0, servers.length);

        ready();

        // if we're hot-reloading, we aren't actually stopping; don't emit
        if (!hot) {
          frock.emit('stop');
        }
      }
    }
  }
}

function noop() {
  // nooperations
}