'use strict';

/**
 * Copyright 2015 Urban Airship Inc. and Contributors. Subject to the LICENSE
 * file at the top-level directory of this distribution and at
 * https://github.com/urbanairship/frock/blob/master/LICENSE
 */
var bole = require('bole');
var createDeter = require('deter');

var createHandlerRegister = require('./register-handler');

var log = bole('frock/middleware');

module.exports = {
  processMiddleware: processMiddleware,
  noopMiddleware: noopMiddleware,
  handleServerError: handleServerError,
  dup: dup,
  createConnectionFilter: createConnectionFilter
};

function processMiddleware(frock, logger) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var middlewares = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];
  var route = arguments[4];

  var handlers = createHandlerRegister(frock.pwd);

  var middlewareStack = middlewares.map(function (middleware) {
    if (typeof middleware.handler === 'function') {
      log.debug('initializing non-required middleware');
      return middleware.handler(frock, logger, middleware.options);
    }

    log.debug('initializing middleware ' + middleware.handler);

    var handler = void 0;

    try {
      handler = handlers.register(middleware.handler);
    } catch (e) {
      log.error('Error registering middleware ' + e.toString(), middleware);

      return noopMiddleware(middleware.handler);
    }

    return handler(frock, logger, middleware.options);
  });

  handleRequest.end = end;

  return handleRequest;

  function handleRequest(req, res) {
    mw(0, req, res);

    function mw(idx, rq, rs) {
      if (idx >= middlewareStack.length) {
        return route(rq, rs);
      }

      middlewareStack[idx](rq, rs, mw.bind(null, ++idx));
    }
  }

  function end() {
    handlers.destroy();
  }
}

function noopMiddleware(missingName) {
  return handler;

  function handler(req, res, next) {
    log.error('no-op middleware: ' + missingName + ' was not found');

    next(req, res);
  }
}

function handleServerError(logger, config) {
  return onError;

  function onError(err) {
    if (err.code === 'EADDRINUSE') {
      logger.error('port ' + config.port + ' could not be bound, address in use');

      return;
    }

    logger.error('server running on ' + config.port + ' encountered an error: ' + err + '--' + 'it\'s recommended that you restart frock');
  }
}

function dup(obj) {
  return JSON.parse(JSON.stringify(obj));
}

function createConnectionFilter() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$argv = _ref.argv,
      argv = _ref$argv === undefined ? {} : _ref$argv;

  var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var globalConfig = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var fail = arguments[3];

  var disableWhitelist = argv['unsafe-disable-connection-filtering'];

  if (disableWhitelist) {
    // return a noop router
    return function (router) {
      return function () {
        return router.apply(undefined, arguments);
      };
    };
  }

  var constraints = config.connection || {};

  if (!constraints.whitelist && !constraints.blacklist) {
    constraints = globalConfig.connection;
  }

  return createDeter(constraints, fail);
}